/* tarot implements the rules of the tarot game
 * Copyright (C) 2019  Vivien Kraus
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>
#include <tarot/cnn_private.h>
#include <tarot/game_private.h>
#include <tarot/features_private.h>
#include <stdlib.h>
#include <assert.h>
#include "xalloc.h"
#include "gettext.h"
#define _(String) dgettext (PACKAGE, String)

size_t
tarot_cnn_predict (const TarotGame * game, size_t max_score, double *score)
{
  return cnn_predict (game, max_score, score);
}

void
tarot_cnn_predict_alloc (const TarotGame * game, double **score,
                         size_t *n_score)
{
  size_t n = cnn_predict (game, 0, NULL);
  *score = xmalloc (n * sizeof (double));
  n = cnn_predict (game, *n_score, *score);
  assert (n == *n_score);
}

size_t
tarot_cnn_predictor_construct (size_t max_mem, char *mem, size_t *alignment)
{
  return cnn_predictor_construct (max_mem, mem, alignment);
}

TarotPredictor *
tarot_cnn_predictor_alloc (void)
{
  size_t alignment;
  size_t required = cnn_predictor_construct (0, NULL, &alignment);
  size_t unused;
  void *mem = xmalloc (required);
  assert (alignment >= 1 && alignment <= alignof (max_align_t));
  unused = cnn_predictor_construct (required, mem, &alignment);
  assert (unused == required);
  (void) unused;
  return mem;
}

#include <tarot/cnn_private_impl.h>
#include <tarot/game_private_impl.h>
#include <tarot/features_private_impl.h>
