/* tarot implements the rules of the tarot game
 * Copyright (C) 2019  Vivien Kraus
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef H_TAROT_CNN_INCLUDED
#define H_TAROT_CNN_INCLUDED

#include <stddef.h>

#ifdef __cplusplus
extern "C"
{
#endif

  struct TarotGame;
  struct TarotPredictor;

  /**
   * tarot_cnn_predict:
   * @score_out: (array length=max_score):
   */
  size_t tarot_cnn_predict (const TarotGame * game, size_t max_score,
                            double *score_out);

  /**
   * tarot_cnn_predict_alloc:
   * @score: (out) (array length=n_score):
   * @n_score: (out):
   */
  void tarot_cnn_predict_alloc (const TarotGame * game, double **score,
                                size_t *n_score);

  /**
   * tarot_cnn_predictor_construct:
   * @mem_out: (array length=max_mem):
   * @alignment: (out):
   */
  size_t tarot_cnn_predictor_construct (size_t max_mem, char *mem_out,
                                        size_t *alignment);

  /**
   * tarot_cnn_predictor_alloc:
   */
  struct TarotPredictor *tarot_cnn_predictor_alloc (void);

#ifdef __cplusplus
}
#endif                          /* __cplusplus */

#endif                          /* not H_TAROT_CNN_INCLUDED */
