/* tarot implements the rules of the tarot game
 * Copyright (C) 2018, 2019  Vivien Kraus
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>
#include <tarot/xml.h>
#include <tarot/mcts_private.h>
#include "xalloc.h"
#include <string.h>

#ifdef __FRAMAC__
#undef alignof
#define alignof(x) (16)
#endif /* __FRAMAC__ */

size_t
tarot_mcts_construct (size_t max_mem, char *mem, size_t *alignment,
                      size_t max_pool)
{
  return mcts_construct (max_mem, mem, alignment, max_pool);
}

TarotMcts *
tarot_mcts_alloc (size_t max)
{
  size_t alignment;
  size_t required = mcts_construct (0, NULL, &alignment, max);
  size_t unused;
  void *mem = xmalloc (required);
  assert (alignment >= 1 && alignment <= alignof (max_align_t));
  unused = mcts_construct (required, mem, &alignment, max);
  assert (unused == required);
  (void) unused;
  return mem;
}

void
tarot_mcts_copy (TarotMcts * dest, const TarotMcts * source)
{
  mcts_copy (dest, source);
}

TarotMcts *
tarot_mcts_dup (size_t new_max_mem, const TarotMcts * source)
{
  TarotMcts *ret = tarot_mcts_alloc (new_max_mem);
  mcts_copy (ret, source);
  return ret;
}

void
tarot_mcts_free (TarotMcts * mcts)
{
  free (mcts);
}

TarotMctsError
tarot_mcts_set_base (TarotMcts * mcts, const TarotGame * base)
{
  return mcts_set_base (mcts, base);
}

void
tarot_mcts_seed (TarotMcts * mcts, size_t seed_size, const void *seed)
{
  mcts_seed (mcts, seed_size, seed);
}

TarotMctsError
tarot_mcts_run (TarotMcts * mcts, size_t n_iterations)
{
  return mcts_run (mcts, n_iterations);
}

const TarotGameEvent *
tarot_mcts_get_best (const TarotMcts * mcts)
{
  return mcts_get_best (mcts);
}

#include <tarot/mcts_private_impl.h>
