/* tarot implements the rules of the tarot game
 * Copyright (C) 2018, 2019  Vivien Kraus
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef H_TAROT_MCTS_INCLUDED
#define H_TAROT_MCTS_INCLUDED
#include <stddef.h>
#include <tarot/player.h>
#ifdef __cplusplus
extern "C"
{
#endif                          /* __cplusplus */

  struct TarotMcts;
  typedef struct TarotMcts TarotMcts;

  struct TarotGameEvent;

  struct TarotGame;

  /**
   * tarot_mcts_construct:
   * @mem_out: (array length=max_mem):
   * @alignment: (out):
   */
  size_t tarot_mcts_construct (size_t max_mem, char *mem_out,
                               size_t *alignment, size_t max_pool);

  void tarot_mcts_copy (TarotMcts * dest, const TarotMcts * source);

  /**
   * tarot_mcts_alloc: (constructor)
   */
  TarotMcts *tarot_mcts_alloc (size_t max_mem);

  TarotMcts *tarot_mcts_dup (size_t new_max_mem, const TarotMcts * source);

  void tarot_mcts_free (TarotMcts * mcts);

  typedef enum
  {
    TAROT_MCTS_OK = 0,
    TAROT_MCTS_NONEXT,
    TAROT_MCTS_UNKNOWNNEXT,
    TAROT_MCTS_NODATA,
    TAROT_MCTS_NOMEM
  } TarotMctsError;

  /**
   * tarot_mcts_set_base:
   * @base: (type TarotGame):
   */
  TarotMctsError tarot_mcts_set_base (TarotMcts * mcts,
                                      const struct TarotGame *base);

  /**
   * tarot_mcts_seed:
   * @seed: (array length=seed_size) (element-type char):
   */
  void tarot_mcts_seed (TarotMcts * mcts, size_t seed_size, const void *seed);

  /**
   * tarot_mcts_run:
   */
  TarotMctsError tarot_mcts_run (TarotMcts * mcts, size_t n_iterations);

  /**
   * tarot_mcts_get_best:
   * @returns: (type Tarot.GameEvent) (transfer none):
   */
  const struct TarotGameEvent *tarot_mcts_get_best (const TarotMcts * mcts);

#ifdef __cplusplus
}
#endif                          /* __cplusplus */

#endif                          /* not H_TAROT_MCTS_INCLUDED */

/* Local Variables: */
/* mode: c */
/* End: */
