#ifndef HAVE_CONFIG_H
#include <config.h>
#endif /* HAVE_CONFIG_H */

#include <tarot.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

int
main ()
{
  tarot_set_datadir (DATADIR);
  if (tarot_init (LOCALEDIR) != 0)
    {
      fprintf (stderr, "Error: could not load libtarot.\n");
      return EXIT_FAILURE;
    }
  {
    TarotGame *game = tarot_game_alloc ();
    TarotGameEvent *event = NULL;
    (void) event;
    event = tarot_game_event_alloc_setup (5, 1);
    if (tarot_game_add_event (game, event) != TAROT_GAME_OK)
      {
        fprintf (stderr, "%s:%d: Error: could not add the event.", __FILE__, __LINE__);
        return EXIT_FAILURE;
      }
    tarot_game_event_free (event);
    {
      const TarotPlayer player = tarot_string_to_player_c ("P5");
      TarotCard cards[15];
      size_t i = 0;
      assert (player != ((TarotPlayer) (-1)));
      cards[i] = tarot_string_to_card_c ("CH");
      assert (cards[i] != ((TarotCard) (-1)));
      i++;
      cards[i] = tarot_string_to_card_c ("QH");
      assert (cards[i] != ((TarotCard) (-1)));
      i++;
      cards[i] = tarot_string_to_card_c ("KH");
      assert (cards[i] != ((TarotCard) (-1)));
      i++;
      cards[i] = tarot_string_to_card_c ("8C");
      assert (cards[i] != ((TarotCard) (-1)));
      i++;
      cards[i] = tarot_string_to_card_c ("AD");
      assert (cards[i] != ((TarotCard) (-1)));
      i++;
      cards[i] = tarot_string_to_card_c ("2D");
      assert (cards[i] != ((TarotCard) (-1)));
      i++;
      cards[i] = tarot_string_to_card_c ("QD");
      assert (cards[i] != ((TarotCard) (-1)));
      i++;
      cards[i] = tarot_string_to_card_c ("KD");
      assert (cards[i] != ((TarotCard) (-1)));
      i++;
      cards[i] = tarot_string_to_card_c ("1T");
      assert (cards[i] != ((TarotCard) (-1)));
      i++;
      cards[i] = tarot_string_to_card_c ("4T");
      assert (cards[i] != ((TarotCard) (-1)));
      i++;
      cards[i] = tarot_string_to_card_c ("8T");
      assert (cards[i] != ((TarotCard) (-1)));
      i++;
      cards[i] = tarot_string_to_card_c ("9T");
      assert (cards[i] != ((TarotCard) (-1)));
      i++;
      cards[i] = tarot_string_to_card_c ("13T");
      assert (cards[i] != ((TarotCard) (-1)));
      i++;
      cards[i] = tarot_string_to_card_c ("20T");
      assert (cards[i] != ((TarotCard) (-1)));
      i++;
      cards[i] = tarot_string_to_card_c ("21T");
      assert (cards[i] != ((TarotCard) (-1)));
      i++;
      event = tarot_game_event_alloc_deal (player, sizeof (cards) / sizeof (cards[0]), cards);
    }
    if (tarot_game_add_event (game, event) != TAROT_GAME_OK)
      {
        fprintf (stderr, "%s:%d: Error: could not add the event.", __FILE__, __LINE__);
        return EXIT_FAILURE;
      }
    tarot_game_event_free (event);
    event = tarot_game_event_alloc_bid (TAROT_TAKE);
    if (tarot_game_add_event (game, event) != TAROT_GAME_OK)
      {
        fprintf (stderr, "%s:%d: Error: could not add the event.", __FILE__, __LINE__);
        return EXIT_FAILURE;
      }
    tarot_game_event_free (event);
    event = tarot_game_event_alloc_bid (TAROT_PASS);
    if (tarot_game_add_event (game, event) != TAROT_GAME_OK)
      {
        fprintf (stderr, "%s:%d: Error: could not add the event.", __FILE__, __LINE__);
        return EXIT_FAILURE;
      }
    tarot_game_event_free (event);
    event = tarot_game_event_alloc_bid (TAROT_PASS);
    if (tarot_game_add_event (game, event) != TAROT_GAME_OK)
      {
        fprintf (stderr, "%s:%d: Error: could not add the event.", __FILE__, __LINE__);
        return EXIT_FAILURE;
      }
    tarot_game_event_free (event);
    event = tarot_game_event_alloc_bid (TAROT_PASS);
    if (tarot_game_add_event (game, event) != TAROT_GAME_OK)
      {
        fprintf (stderr, "%s:%d: Error: could not add the event.", __FILE__, __LINE__);
        return EXIT_FAILURE;
      }
    tarot_game_event_free (event);
    event = tarot_game_event_alloc_bid (TAROT_PUSH);
    if (tarot_game_add_event (game, event) != TAROT_GAME_OK)
      {
        fprintf (stderr, "%s:%d: Error: could not add the event.", __FILE__, __LINE__);
        return EXIT_FAILURE;
      }
    tarot_game_event_free (event);
    event = tarot_game_event_alloc_decl (1);
    if (tarot_game_add_event (game, event) != TAROT_GAME_OK)
      {
        fprintf (stderr, "%s:%d: Error: could not add the event.", __FILE__, __LINE__);
        return EXIT_FAILURE;
      }
    tarot_game_event_free (event);
    {
      TarotCard call = tarot_string_to_card_c ("KC");
      assert (call != ((TarotCard) (-1)));
      event = tarot_game_event_alloc_call (call);
    }
    if (tarot_game_add_event (game, event) != TAROT_GAME_OK)
      {
        fprintf (stderr, "%s:%d: Error: could not add the event.", __FILE__, __LINE__);
        return EXIT_FAILURE;
      }
    tarot_game_event_free (event);
    /* Now running the tests */
    assert (tarot_game_step (game) == TAROT_DOG);
    assert (tarot_game_n_players (game) == 5);
    assert (tarot_game_with_call (game));
    assert (tarot_game_n_tricks (game) == 15);
    {
      TarotPlayer next;
      assert (tarot_game_get_next (game, &next) == TAROT_GAME_NA);
    }
    {
      TarotPlayer p;
      assert (tarot_game_get_main_player (game, &p) == TAROT_GAME_OK);
      assert (p != ((TarotPlayer) (-1)));
      assert (p == tarot_string_to_player_c ("P5"));
    }
    {
      size_t n;
      size_t i = 0;
      TarotCard c;
      TarotPlayer who = tarot_string_to_player_c ("P5");
      assert (who != ((TarotPlayer) (-1)));
      assert (tarot_game_get_deal_of (game, who, &n, i++, 1, &c) == TAROT_GAME_OK);
      assert (n == 15);
      assert (c != ((TarotCard) (-1)));
      assert (c == tarot_string_to_card_c ("CH"));
      assert (tarot_game_get_deal_of (game, who, &n, i++, 1, &c) == TAROT_GAME_OK);
      assert (n == 15);
      assert (c != ((TarotCard) (-1)));
      assert (c == tarot_string_to_card_c ("QH"));
      assert (tarot_game_get_deal_of (game, who, &n, i++, 1, &c) == TAROT_GAME_OK);
      assert (n == 15);
      assert (c != ((TarotCard) (-1)));
      assert (c == tarot_string_to_card_c ("KH"));
      assert (tarot_game_get_deal_of (game, who, &n, i++, 1, &c) == TAROT_GAME_OK);
      assert (n == 15);
      assert (c != ((TarotCard) (-1)));
      assert (c == tarot_string_to_card_c ("8C"));
      assert (tarot_game_get_deal_of (game, who, &n, i++, 1, &c) == TAROT_GAME_OK);
      assert (n == 15);
      assert (c != ((TarotCard) (-1)));
      assert (c == tarot_string_to_card_c ("AD"));
      assert (tarot_game_get_deal_of (game, who, &n, i++, 1, &c) == TAROT_GAME_OK);
      assert (n == 15);
      assert (c != ((TarotCard) (-1)));
      assert (c == tarot_string_to_card_c ("2D"));
      assert (tarot_game_get_deal_of (game, who, &n, i++, 1, &c) == TAROT_GAME_OK);
      assert (n == 15);
      assert (c != ((TarotCard) (-1)));
      assert (c == tarot_string_to_card_c ("QD"));
      assert (tarot_game_get_deal_of (game, who, &n, i++, 1, &c) == TAROT_GAME_OK);
      assert (n == 15);
      assert (c != ((TarotCard) (-1)));
      assert (c == tarot_string_to_card_c ("KD"));
      assert (tarot_game_get_deal_of (game, who, &n, i++, 1, &c) == TAROT_GAME_OK);
      assert (n == 15);
      assert (c != ((TarotCard) (-1)));
      assert (c == tarot_string_to_card_c ("1T"));
      assert (tarot_game_get_deal_of (game, who, &n, i++, 1, &c) == TAROT_GAME_OK);
      assert (n == 15);
      assert (c != ((TarotCard) (-1)));
      assert (c == tarot_string_to_card_c ("4T"));
      assert (tarot_game_get_deal_of (game, who, &n, i++, 1, &c) == TAROT_GAME_OK);
      assert (n == 15);
      assert (c != ((TarotCard) (-1)));
      assert (c == tarot_string_to_card_c ("8T"));
      assert (tarot_game_get_deal_of (game, who, &n, i++, 1, &c) == TAROT_GAME_OK);
      assert (n == 15);
      assert (c != ((TarotCard) (-1)));
      assert (c == tarot_string_to_card_c ("9T"));
      assert (tarot_game_get_deal_of (game, who, &n, i++, 1, &c) == TAROT_GAME_OK);
      assert (n == 15);
      assert (c != ((TarotCard) (-1)));
      assert (c == tarot_string_to_card_c ("13T"));
      assert (tarot_game_get_deal_of (game, who, &n, i++, 1, &c) == TAROT_GAME_OK);
      assert (n == 15);
      assert (c != ((TarotCard) (-1)));
      assert (c == tarot_string_to_card_c ("20T"));
      assert (tarot_game_get_deal_of (game, who, &n, i++, 1, &c) == TAROT_GAME_OK);
      assert (n == 15);
      assert (c != ((TarotCard) (-1)));
      assert (c == tarot_string_to_card_c ("21T"));
    }
    {
      size_t n;
      size_t i = 0;
      TarotCard c;
      TarotPlayer who = tarot_string_to_player_c ("P5");
      assert (who != ((TarotPlayer) (-1)));
      assert (tarot_game_get_cards (game, who, &n, i++, 1, &c) == TAROT_GAME_OK);
      assert (n == 15);
      assert (c != ((TarotCard) (-1)));
      assert (c == tarot_string_to_card_c ("CH"));
      assert (tarot_game_get_cards (game, who, &n, i++, 1, &c) == TAROT_GAME_OK);
      assert (n == 15);
      assert (c != ((TarotCard) (-1)));
      assert (c == tarot_string_to_card_c ("QH"));
      assert (tarot_game_get_cards (game, who, &n, i++, 1, &c) == TAROT_GAME_OK);
      assert (n == 15);
      assert (c != ((TarotCard) (-1)));
      assert (c == tarot_string_to_card_c ("KH"));
      assert (tarot_game_get_cards (game, who, &n, i++, 1, &c) == TAROT_GAME_OK);
      assert (n == 15);
      assert (c != ((TarotCard) (-1)));
      assert (c == tarot_string_to_card_c ("8C"));
      assert (tarot_game_get_cards (game, who, &n, i++, 1, &c) == TAROT_GAME_OK);
      assert (n == 15);
      assert (c != ((TarotCard) (-1)));
      assert (c == tarot_string_to_card_c ("AD"));
      assert (tarot_game_get_cards (game, who, &n, i++, 1, &c) == TAROT_GAME_OK);
      assert (n == 15);
      assert (c != ((TarotCard) (-1)));
      assert (c == tarot_string_to_card_c ("2D"));
      assert (tarot_game_get_cards (game, who, &n, i++, 1, &c) == TAROT_GAME_OK);
      assert (n == 15);
      assert (c != ((TarotCard) (-1)));
      assert (c == tarot_string_to_card_c ("QD"));
      assert (tarot_game_get_cards (game, who, &n, i++, 1, &c) == TAROT_GAME_OK);
      assert (n == 15);
      assert (c != ((TarotCard) (-1)));
      assert (c == tarot_string_to_card_c ("KD"));
      assert (tarot_game_get_cards (game, who, &n, i++, 1, &c) == TAROT_GAME_OK);
      assert (n == 15);
      assert (c != ((TarotCard) (-1)));
      assert (c == tarot_string_to_card_c ("1T"));
      assert (tarot_game_get_cards (game, who, &n, i++, 1, &c) == TAROT_GAME_OK);
      assert (n == 15);
      assert (c != ((TarotCard) (-1)));
      assert (c == tarot_string_to_card_c ("4T"));
      assert (tarot_game_get_cards (game, who, &n, i++, 1, &c) == TAROT_GAME_OK);
      assert (n == 15);
      assert (c != ((TarotCard) (-1)));
      assert (c == tarot_string_to_card_c ("8T"));
      assert (tarot_game_get_cards (game, who, &n, i++, 1, &c) == TAROT_GAME_OK);
      assert (n == 15);
      assert (c != ((TarotCard) (-1)));
      assert (c == tarot_string_to_card_c ("9T"));
      assert (tarot_game_get_cards (game, who, &n, i++, 1, &c) == TAROT_GAME_OK);
      assert (n == 15);
      assert (c != ((TarotCard) (-1)));
      assert (c == tarot_string_to_card_c ("13T"));
      assert (tarot_game_get_cards (game, who, &n, i++, 1, &c) == TAROT_GAME_OK);
      assert (n == 15);
      assert (c != ((TarotCard) (-1)));
      assert (c == tarot_string_to_card_c ("20T"));
      assert (tarot_game_get_cards (game, who, &n, i++, 1, &c) == TAROT_GAME_OK);
      assert (n == 15);
      assert (c != ((TarotCard) (-1)));
      assert (c == tarot_string_to_card_c ("21T"));
    }
    {
      size_t n;
      size_t i = 0;
      TarotBid b;
      assert (tarot_game_get_bids (game, &n, i++, 1, &b) == TAROT_GAME_OK);
      assert (n == 5);
      assert (b == TAROT_TAKE);
      assert (tarot_game_get_bids (game, &n, i++, 1, &b) == TAROT_GAME_OK);
      assert (n == 5);
      assert (b == TAROT_PASS);
      assert (tarot_game_get_bids (game, &n, i++, 1, &b) == TAROT_GAME_OK);
      assert (n == 5);
      assert (b == TAROT_PASS);
      assert (tarot_game_get_bids (game, &n, i++, 1, &b) == TAROT_GAME_OK);
      assert (n == 5);
      assert (b == TAROT_PASS);
      assert (tarot_game_get_bids (game, &n, i++, 1, &b) == TAROT_GAME_OK);
      assert (n == 5);
      assert (b == TAROT_PUSH);
    }
    {
      TarotPlayer taker;
      assert (tarot_game_get_taker (game, &taker) == TAROT_GAME_OK);
      assert (taker != ((TarotPlayer) (-1)));
      assert (taker == tarot_string_to_player_c ("P5"));
    }
    {
      size_t n;
      size_t i = 0;
      int d;
      assert (tarot_game_get_declarations (game, &n, i++, 1, &d) == TAROT_GAME_OK);
      assert (n == 5);
      assert (d);
      assert (tarot_game_get_declarations (game, &n, i++, 1, &d) == TAROT_GAME_OK);
      assert (n == 5);
      assert (!d);
      assert (tarot_game_get_declarations (game, &n, i++, 1, &d) == TAROT_GAME_OK);
      assert (n == 5);
      assert (!d);
      assert (tarot_game_get_declarations (game, &n, i++, 1, &d) == TAROT_GAME_OK);
      assert (n == 5);
      assert (!d);
      assert (tarot_game_get_declarations (game, &n, i++, 1, &d) == TAROT_GAME_OK);
      assert (n == 5);
      assert (!d);
    }
    {
      TarotPlayer declarant;
      assert (tarot_game_get_declarant (game, &declarant) == TAROT_GAME_OK);
      assert (declarant != ((TarotPlayer) (-1)));
      assert (declarant == tarot_string_to_player_c ("P1"));
    }
    {
      TarotCard call;
      assert (tarot_game_get_call (game, &call) == TAROT_GAME_OK);
      assert (call != (TarotCard) (-1));
      assert (call == tarot_string_to_card_c ("KC"));
    }
    {
      TarotPlayer partner;
      assert (tarot_game_get_partner (game, &partner) == TAROT_GAME_NA);
    }
    {
      size_t n;
      assert (tarot_game_get_dog (game, &n, 0, 0, NULL) == TAROT_GAME_NA);
      assert (n == 0);
    }
    {
      size_t n;
      assert (tarot_game_get_full_discard (game, &n, 0, 0, NULL) == TAROT_GAME_NA);
      assert (n == 0);
    }
    {
      size_t n;
      assert (tarot_game_get_public_discard (game, &n, 0, 0, NULL) == TAROT_GAME_NA);
      assert (n == 0);
    }
    {
      size_t n;
      int size;
      TarotPlayer who = tarot_string_to_player_c ("P1");
      assert (who != ((TarotPlayer) (-1)));
      assert (tarot_game_get_handful (game, who, &size, &n, 0, 0, NULL) == TAROT_GAME_NA);
      assert (n == 0);
    }
    {
      size_t n;
      int size;
      TarotPlayer who = tarot_string_to_player_c ("P2");
      assert (who != ((TarotPlayer) (-1)));
      assert (tarot_game_get_handful (game, who, &size, &n, 0, 0, NULL) == TAROT_GAME_NA);
      assert (n == 0);
    }
    {
      size_t n;
      int size;
      TarotPlayer who = tarot_string_to_player_c ("P3");
      assert (who != ((TarotPlayer) (-1)));
      assert (tarot_game_get_handful (game, who, &size, &n, 0, 0, NULL) == TAROT_GAME_NA);
      assert (n == 0);
    }
    {
      size_t n;
      int size;
      TarotPlayer who = tarot_string_to_player_c ("P4");
      assert (who != ((TarotPlayer) (-1)));
      assert (tarot_game_get_handful (game, who, &size, &n, 0, 0, NULL) == TAROT_GAME_NA);
      assert (n == 0);
    }
    {
      size_t trick;
      assert (tarot_game_get_current_trick (game, &trick) == TAROT_GAME_NA);
      assert (trick == 0);
    }
    {
      TarotSuit lead_suit;
      assert (tarot_game_get_lead_suit (game, 0, &lead_suit) == TAROT_GAME_NA);
    }
    {
      TarotSuit lead_suit;
      assert (tarot_game_get_lead_suit (game, 1, &lead_suit) == TAROT_GAME_NA);
    }
    {
      TarotSuit lead_suit;
      assert (tarot_game_get_lead_suit (game, 14, &lead_suit) == TAROT_GAME_NA);
    }
    {
      TarotNumber max_trump;
      assert (tarot_game_get_max_trump (game, 0, &max_trump) == TAROT_GAME_NA);
    }
    {
      TarotNumber max_trump;
      assert (tarot_game_get_max_trump (game, 1, &max_trump) == TAROT_GAME_NA);
    }
    {
      TarotNumber max_trump;
      assert (tarot_game_get_max_trump (game, 14, &max_trump) == TAROT_GAME_NA);
    }
    {
      TarotPlayer trick_leader;
      assert (tarot_game_get_trick_leader (game, 0, &trick_leader) == TAROT_GAME_NA);
    }
    {
      TarotPlayer trick_leader;
      assert (tarot_game_get_trick_leader (game, 1, &trick_leader) == TAROT_GAME_NA);
    }
    {
      TarotPlayer trick_leader;
      assert (tarot_game_get_trick_leader (game, 14, &trick_leader) == TAROT_GAME_NA);
    }
    {
      TarotPlayer trick_taker;
      assert (tarot_game_get_trick_taker (game, 0, &trick_taker) == TAROT_GAME_NA);
      assert (trick_taker != ((TarotPlayer) (-1)));
      assert (trick_taker == tarot_string_to_player_c ("P6"));
    }
    {
      TarotPlayer trick_taker;
      assert (tarot_game_get_trick_taker (game, 1, &trick_taker) == TAROT_GAME_NA);
      assert (trick_taker != ((TarotPlayer) (-1)));
      assert (trick_taker == tarot_string_to_player_c ("P6"));
    }
    {
      TarotPlayer trick_taker;
      assert (tarot_game_get_trick_taker (game, 14, &trick_taker) == TAROT_GAME_NA);
      assert (trick_taker != ((TarotPlayer) (-1)));
      assert (trick_taker == tarot_string_to_player_c ("P6"));
    }
    {
      size_t n_cards;
      assert (tarot_game_get_trick_cards (game, 0, &n_cards, 0, 0, NULL) == TAROT_GAME_NA);
    }
    {
      size_t n_cards;
      assert (tarot_game_get_trick_cards (game, 1, &n_cards, 0, 0, NULL) == TAROT_GAME_NA);
    }
    {
      size_t n_cards;
      assert (tarot_game_get_trick_cards (game, 14, &n_cards, 0, 0, NULL) == TAROT_GAME_NA);
    }
    {
      unsigned int hp;
      unsigned int od;
      assert (tarot_game_get_points_in_trick (game, 0, &hp, &od) == TAROT_GAME_NA);
    }
    {
      unsigned int hp;
      unsigned int od;
      assert (tarot_game_get_points_in_trick (game, 1, &hp, &od) == TAROT_GAME_NA);
    }
    {
      unsigned int hp;
      unsigned int od;
      assert (tarot_game_get_points_in_trick (game, 14, &hp, &od) == TAROT_GAME_NA);
    }
    {
      size_t n_players;
      assert (tarot_game_get_scores (game, &n_players, 0, 0, NULL) == TAROT_GAME_NA);
    }
    {
      TarotPlayer next;
      TarotBid min;
      assert (tarot_game_get_hint_bid (game, &next, &min) == TAROT_GAME_NA);
    }
    {
      TarotPlayer next;
      int allowed;
      assert (tarot_game_get_hint_decl (game, &next, &allowed) == TAROT_GAME_NA);
    }
    {
      TarotPlayer next;
      TarotNumber min;
      assert (tarot_game_get_hint_call (game, &next, &min) == TAROT_GAME_NA);
    }
    {
      TarotPlayer next;
      size_t n, n_prio, n_additional;
      assert (tarot_game_get_hint_discard (game, &next, &n, &n_prio, 0, 0, NULL, &n_additional, 0, 0, NULL) == TAROT_GAME_NA);
    }
    {
      TarotPlayer next;
      size_t n_simple, n_double, n_triple, n_prio, n_additional;
      assert (tarot_game_get_hint_handful (game, &next, &n_simple, &n_double, &n_triple, &n_prio, 0, 0, NULL, &n_additional, 0, 0, NULL) == TAROT_GAME_NA);
    }
    {
      TarotPlayer next;
      size_t n_playable;
      assert (tarot_game_get_hint_card (game, &next, &n_playable, 0, 0, NULL) == TAROT_GAME_NA);
    }
    tarot_game_free (game);
  }
  tarot_quit ();
  return EXIT_SUCCESS;
}