/* tarot implements the rules of the tarot game
 * Copyright (C) 2019  Vivien Kraus
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "program_commons.h"

static void parse_options (int argc, char *argv[]);
static void print_row (size_t n, const double *row);

static void
event_handler (const TarotGame * base, TarotPlayer next,
               const TarotGameEvent * event)
{
  (void) next;
  TarotGame *rebuilt = tarot_game_dup (base);
  if (tarot_game_add_event (rebuilt, event) != TAROT_GAME_OK)
    {
      assert (0);
    }
  double *data;
  size_t n;
  tarot_features_cnn_alloc (rebuilt, &data, &n);
  print_row (n, data);
  tarot_game_free (rebuilt);
}

int
main (int argc, char *argv[])
{
  TarotGame *game = NULL;
  tarot_program_setup ();
  parse_options (argc, argv);
  game = tarot_program_read_game ();
  tarot_program_iterate (game, &event_handler);
  return EXIT_SUCCESS;
}

static void
parse_options (int argc, char *argv[])
{
  int c;
  while (1)
    {
      static struct option long_options[] = {
        {N_("help"), no_argument, 0, 'h'},
        {N_("version"), no_argument, 0, 'v'},
        {0, 0, 0, 0}
      };
      int option_index = 0;
      for (option_index = 0; long_options[option_index].name != NULL;
           option_index++)
        {
          long_options[option_index].name =
            _(long_options[option_index].name);
        }
      option_index = 0;
      c = getopt_long (argc, argv, "hv", long_options, &option_index);
      if (c == -1)
        break;
      switch (c)
        {
        case 0:
          break;

        case 'h':
          printf (_
                  ("Usage: tarot-cnn-features [OPTIONS]... < GAME > FEATURES\n\
Extract the tensor-shaped game features.  See the info manual for the\n\
details.  Options:\n\
- -h, --help: print this message and exit;\n\
- -v, --version: print the program version and exit.\n\
"));
          exit (EXIT_SUCCESS);
          break;

        case 'v':
          printf (_("%s (libtarot %s)\n\
\n\
Copyright © 2019 Vivien Kraus\nThis is free software; see the source for copying conditions.  There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n"), "tarot-cnn-features", tarot_libtarot_version ());
          exit (EXIT_SUCCESS);
          break;

        case '?':
          /* getopt_long already printed an error message. */
          exit (EXIT_FAILURE);
          break;

        default:
          abort ();
        }
    }
  if (optind < argc)
    {
      fprintf (stderr, _("Error: extra arguments:\n"));
      while (optind < argc)
        fprintf (stderr, _("- %s\n"), argv[optind++]);
      exit (EXIT_FAILURE);
    }
}

static void
print_row (size_t n, const double *data)
{
  size_t i;
  for (i = 0; i < n; i++)
    {
      if (i != 0)
        {
          printf (",");
        }
      if (data[i] == 0.0)
        {
          printf ("0");
        }
      else if (data[i] == 1.0)
        {
          printf ("1");
        }
      else
        {
          printf ("%f", data[i]);
        }
    }
  printf ("\n");
}
